/****************************************************************************
**
** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt3D module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QGLBEZIERPATCHES_H
#define QGLBEZIERPATCHES_H

#include <Qt3D/qvector2darray.h>
#include <Qt3D/qvector3darray.h>

#include <QtCore/qscopedpointer.h>

QT_BEGIN_NAMESPACE

class QGLBezierPatchesPrivate;
class QGLBuilder;
class QRay3D;

class Q_QT3D_EXPORT QGLBezierPatches
{
public:
    QGLBezierPatches();
    QGLBezierPatches(const QGLBezierPatches &other);
    virtual ~QGLBezierPatches();

    QGLBezierPatches &operator=(const QGLBezierPatches &other);

    QVector3DArray positions() const;
    void setPositions(const QVector3DArray &positions);

    QVector2DArray textureCoords() const;
    void setTextureCoords(const QVector2DArray &textureCoords);

    int subdivisionDepth() const;
    void setSubdivisionDepth(int value);

    void transform(const QMatrix4x4 &matrix);
    QGLBezierPatches transformed(const QMatrix4x4 &matrix) const;

    bool intersects(const QRay3D &ray) const;
    float intersection(const QRay3D &ray, QVector2D *texCoord = 0, int *patch = 0) const;

private:
    QScopedPointer<QGLBezierPatchesPrivate> d_ptr;

    Q_DECLARE_PRIVATE(QGLBezierPatches)

    friend Q_QT3D_EXPORT QGLBuilder &operator<<(QGLBuilder &list, const QGLBezierPatches &patches);
};

Q_QT3D_EXPORT QGLBuilder &operator<<(QGLBuilder &list, const QGLBezierPatches &patches);

QT_END_NAMESPACE

#endif
