
if (CMAKE_VERSION VERSION_LESS 2.8.3)
    message(FATAL_ERROR "Qt 5 requires at least CMake version 2.8.3")
endif()

get_filename_component(_qt53D_install_prefix "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

set(Qt53D_VERSION_MAJOR 5)
set(Qt53D_VERSION_MINOR 2)
set(Qt53D_VERSION_PATCH 0)
set(Qt53D_VERSION_STRING 5.2.0)

set(Qt53D_LIBRARIES Qt5::3D)

macro(_populate_3D_target_properties Configuration LIB_LOCATION IMPLIB_LOCATION)
    set_property(TARGET Qt5::3D APPEND PROPERTY IMPORTED_CONFIGURATIONS ${Configuration})

    set_target_properties(Qt5::3D PROPERTIES
        "IMPORTED_LINK_INTERFACE_LIBRARIES_${Configuration}" "${_Qt53D_LIB_DEPENDENCIES}"
        "IMPORTED_LOCATION_${Configuration}" "${_qt53D_install_prefix}/lib/${LIB_LOCATION}"
    )
    if(NOT "${IMPLIB_LOCATION}" STREQUAL "")
        set_target_properties(Qt5::3D PROPERTIES
            "IMPORTED_IMPLIB_${Configuration}" "${_qt53D_install_prefix}/lib/${IMPLIB_LOCATION}"
        )
    endif()
endmacro()

if (NOT TARGET Qt5::3D)

    set(Qt53D_INCLUDE_DIRS "${_qt53D_install_prefix}/include/qt5/Qt3D")
    set(Qt53D_PRIVATE_INCLUDE_DIRS
        "${_qt53D_install_prefix}/include/qt5/Qt3D/${Qt53D_VERSION_STRING}"
        "${_qt53D_install_prefix}/include/qt5/Qt3D/${Qt53D_VERSION_STRING}/Qt3D"
    )

    set(Qt53D_DEFINITIONS -DQT_3D_LIB)
    set(Qt53D_COMPILE_DEFINITIONS QT_3D_LIB)

    set(_Qt53D_MODULE_DEPENDENCIES "OpenGL;Gui;Core")

    set(_Qt53D_FIND_DEPENDENCIES_REQUIRED)
    if (Qt53D_FIND_REQUIRED)
        set(_Qt53D_FIND_DEPENDENCIES_REQUIRED REQUIRED)
    endif()
    set(_Qt53D_FIND_DEPENDENCIES_QUIET)
    if (Qt53D_FIND_QUIETLY)
        set(_Qt53D_DEPENDENCIES_FIND_QUIET QUIET)
    endif()

    foreach(_module_dep ${_Qt53D_MODULE_DEPENDENCIES})
        if (NOT Qt5${_module_dep}_FOUND)
            find_package(Qt5${_module_dep}
                ${_Qt53D_DEPENDENCIES_FIND_QUIET}
                ${_Qt53D_FIND_DEPENDENCIES_REQUIRED}
                PATHS "${_qt53D_install_prefix}" NO_DEFAULT_PATH
            )
        endif()

        if (NOT Qt5${_module_dep}_FOUND)
            set(Qt53D_FOUND False)
            return()
        endif()

        list(APPEND Qt53D_INCLUDE_DIRS "${Qt5${_module_dep}_INCLUDE_DIRS}")
        list(APPEND Qt53D_PRIVATE_INCLUDE_DIRS "${Qt5${_module_dep}_PRIVATE_INCLUDE_DIRS}")
        list(APPEND Qt53D_DEFINITIONS ${Qt5${_module_dep}_DEFINITIONS})
        list(APPEND Qt53D_COMPILE_DEFINITIONS ${Qt5${_module_dep}_COMPILE_DEFINITIONS})
        list(APPEND Qt53D_EXECUTABLE_COMPILE_FLAGS ${Qt5${_module_dep}_EXECUTABLE_COMPILE_FLAGS})
    endforeach()
    list(REMOVE_DUPLICATES Qt53D_INCLUDE_DIRS)
    list(REMOVE_DUPLICATES Qt53D_PRIVATE_INCLUDE_DIRS)
    list(REMOVE_DUPLICATES Qt53D_DEFINITIONS)
    list(REMOVE_DUPLICATES Qt53D_COMPILE_DEFINITIONS)
    if (Qt53D_EXECUTABLE_COMPILE_FLAGS)
      list(REMOVE_DUPLICATES Qt53D_EXECUTABLE_COMPILE_FLAGS)
    endif()

    set(_Qt53D_LIB_DEPENDENCIES "Qt5::OpenGL;Qt5::Gui;Qt5::Core")

    add_library(Qt5::3D SHARED IMPORTED)

    _populate_3D_target_properties(RELEASE "Qt53D.dll" "Qt53D.lib" )

    if (EXISTS "${_qt53D_install_prefix}/lib/Qt53Dd.dll" AND EXISTS "${_qt53D_install_prefix}/lib/Qt53Dd.lib" )
        _populate_3D_target_properties(DEBUG "Qt53Dd.dll" "Qt53Dd.lib" )
    endif()





endif()
