
if (CMAKE_VERSION VERSION_LESS 2.8.3)
    message(FATAL_ERROR "Qt 5 requires at least CMake version 2.8.3")
endif()

get_filename_component(_qt53DQuick_install_prefix "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

set(Qt53DQuick_VERSION_MAJOR 5)
set(Qt53DQuick_VERSION_MINOR 2)
set(Qt53DQuick_VERSION_PATCH 0)
set(Qt53DQuick_VERSION_STRING 5.2.0)

set(Qt53DQuick_LIBRARIES Qt5::3DQuick)

macro(_populate_3DQuick_target_properties Configuration LIB_LOCATION IMPLIB_LOCATION)
    set_property(TARGET Qt5::3DQuick APPEND PROPERTY IMPORTED_CONFIGURATIONS ${Configuration})

    set_target_properties(Qt5::3DQuick PROPERTIES
        "IMPORTED_LINK_INTERFACE_LIBRARIES_${Configuration}" "${_Qt53DQuick_LIB_DEPENDENCIES}"
        "IMPORTED_LOCATION_${Configuration}" "${_qt53DQuick_install_prefix}/lib/${LIB_LOCATION}"
    )
    if(NOT "${IMPLIB_LOCATION}" STREQUAL "")
        set_target_properties(Qt5::3DQuick PROPERTIES
            "IMPORTED_IMPLIB_${Configuration}" "${_qt53DQuick_install_prefix}/lib/${IMPLIB_LOCATION}"
        )
    endif()
endmacro()

if (NOT TARGET Qt5::3DQuick)

    set(Qt53DQuick_INCLUDE_DIRS "${_qt53DQuick_install_prefix}/include/qt5/Qt3DQuick")
    set(Qt53DQuick_PRIVATE_INCLUDE_DIRS
        "${_qt53DQuick_install_prefix}/include/qt5/Qt3DQuick/${Qt53DQuick_VERSION_STRING}"
        "${_qt53DQuick_install_prefix}/include/qt5/Qt3DQuick/${Qt53DQuick_VERSION_STRING}/Qt3DQuick"
    )

    set(Qt53DQuick_DEFINITIONS -DQT_3DQUICK_LIB)
    set(Qt53DQuick_COMPILE_DEFINITIONS QT_3DQUICK_LIB)

    set(_Qt53DQuick_MODULE_DEPENDENCIES "3D;Quick;Qml;Gui;Core")

    set(_Qt53DQuick_FIND_DEPENDENCIES_REQUIRED)
    if (Qt53DQuick_FIND_REQUIRED)
        set(_Qt53DQuick_FIND_DEPENDENCIES_REQUIRED REQUIRED)
    endif()
    set(_Qt53DQuick_FIND_DEPENDENCIES_QUIET)
    if (Qt53DQuick_FIND_QUIETLY)
        set(_Qt53DQuick_DEPENDENCIES_FIND_QUIET QUIET)
    endif()

    foreach(_module_dep ${_Qt53DQuick_MODULE_DEPENDENCIES})
        if (NOT Qt5${_module_dep}_FOUND)
            find_package(Qt5${_module_dep}
                ${_Qt53DQuick_DEPENDENCIES_FIND_QUIET}
                ${_Qt53DQuick_FIND_DEPENDENCIES_REQUIRED}
                PATHS "${_qt53DQuick_install_prefix}" NO_DEFAULT_PATH
            )
        endif()

        if (NOT Qt5${_module_dep}_FOUND)
            set(Qt53DQuick_FOUND False)
            return()
        endif()

        list(APPEND Qt53DQuick_INCLUDE_DIRS "${Qt5${_module_dep}_INCLUDE_DIRS}")
        list(APPEND Qt53DQuick_PRIVATE_INCLUDE_DIRS "${Qt5${_module_dep}_PRIVATE_INCLUDE_DIRS}")
        list(APPEND Qt53DQuick_DEFINITIONS ${Qt5${_module_dep}_DEFINITIONS})
        list(APPEND Qt53DQuick_COMPILE_DEFINITIONS ${Qt5${_module_dep}_COMPILE_DEFINITIONS})
        list(APPEND Qt53DQuick_EXECUTABLE_COMPILE_FLAGS ${Qt5${_module_dep}_EXECUTABLE_COMPILE_FLAGS})
    endforeach()
    list(REMOVE_DUPLICATES Qt53DQuick_INCLUDE_DIRS)
    list(REMOVE_DUPLICATES Qt53DQuick_PRIVATE_INCLUDE_DIRS)
    list(REMOVE_DUPLICATES Qt53DQuick_DEFINITIONS)
    list(REMOVE_DUPLICATES Qt53DQuick_COMPILE_DEFINITIONS)
    if (Qt53DQuick_EXECUTABLE_COMPILE_FLAGS)
      list(REMOVE_DUPLICATES Qt53DQuick_EXECUTABLE_COMPILE_FLAGS)
    endif()

    set(_Qt53DQuick_LIB_DEPENDENCIES "Qt5::3D;Qt5::Quick;Qt5::Qml;Qt5::Gui;Qt5::Core")

    add_library(Qt5::3DQuick SHARED IMPORTED)

    _populate_3DQuick_target_properties(RELEASE "Qt53DQuick.dll" "Qt53DQuick.lib" )

    if (EXISTS "${_qt53DQuick_install_prefix}/lib/Qt53DQuickd.dll" AND EXISTS "${_qt53DQuick_install_prefix}/lib/Qt53DQuickd.lib" )
        _populate_3DQuick_target_properties(DEBUG "Qt53DQuickd.dll" "Qt53DQuickd.lib" )
    endif()





endif()
